@@ -31,13 +31,15 @@
 
 #include "grd-clipboard-rdp.h"
 #include "grd-context.h"
-#include "grd-rdp-audio-input.h"
-#include "grd-rdp-audio-playback.h"
 #include "grd-rdp-cursor-renderer.h"
-#include "grd-rdp-display-control.h"
-#include "grd-rdp-dvc.h"
+#include "grd-rdp-dvc-audio-input.h"
+#include "grd-rdp-dvc-audio-playback.h"
+#include "grd-rdp-dvc-display-control.h"
+#include "grd-rdp-dvc-graphics-pipeline.h"
+#include "grd-rdp-dvc-handler.h"
+#include "grd-rdp-dvc-input.h"
+#include "grd-rdp-dvc-telemetry.h"
 #include "grd-rdp-event-queue.h"
-#include "grd-rdp-graphics-pipeline.h"
 #include "grd-rdp-layout-manager.h"
 #include "grd-rdp-network-autodetection.h"
 #include "grd-rdp-private.h"
@@ -45,7 +47,6 @@
 #include "grd-rdp-sam.h"
 #include "grd-rdp-server.h"
 #include "grd-rdp-session-metrics.h"
-#include "grd-rdp-telemetry.h"
 #include "grd-settings.h"
 
 #define MAX_MONITOR_COUNT_HEADLESS 16
@@ -449,6 +450,12 @@ grd_session_rdp_tear_down_channel (GrdSessionRdp *session_rdp,
     case GRD_RDP_CHANNEL_DISPLAY_CONTROL:
       g_clear_object (&rdp_peer_context->display_control);
       break;
+    case GRD_RDP_CHANNEL_GRAPHICS_PIPELINE:
+      g_assert_not_reached ();
+      break;
+    case GRD_RDP_CHANNEL_INPUT:
+      g_clear_object (&rdp_peer_context->input);
+      break;
     case GRD_RDP_CHANNEL_TELEMETRY:
       g_clear_object (&rdp_peer_context->telemetry);
       break;
@@ -535,7 +542,7 @@ rdp_input_mouse_event (rdpInput *rdp_input,
   RdpPeerContext *rdp_peer_context = (RdpPeerContext *) rdp_input->context;
   GrdSessionRdp *session_rdp = rdp_peer_context->session_rdp;
   GrdRdpEventQueue *rdp_event_queue = session_rdp->rdp_event_queue;
-  GrdEventPointerMotionAbs motion_abs = {};
+  GrdEventMotionAbs motion_abs = {};
   GrdStream *stream = NULL;
   GrdButtonState button_state;
   int32_t button = 0;
@@ -637,6 +644,43 @@ rdp_input_extended_mouse_event (rdpInput *rdp_input,
   return TRUE;
 }
 
+static BOOL
+rdp_input_rel_mouse_event (rdpInput *rdp_input,
+                           uint16_t  flags,
+                           int16_t   dx,
+                           int16_t   dy)
+{
+  RdpPeerContext *rdp_peer_context = (RdpPeerContext *) rdp_input->context;
+  GrdSessionRdp *session_rdp = rdp_peer_context->session_rdp;
+  GrdRdpEventQueue *rdp_event_queue = session_rdp->rdp_event_queue;
+  GrdButtonState button_state;
+  int32_t button = 0;
+
+  grd_rdp_event_queue_add_input_event_pointer_motion (rdp_event_queue, dx, dy);
+
+  button_state = flags & PTR_FLAGS_DOWN ? GRD_BUTTON_STATE_PRESSED
+                                        : GRD_BUTTON_STATE_RELEASED;
+
+  if (flags & PTR_FLAGS_BUTTON1)
+    button = BTN_LEFT;
+  else if (flags & PTR_FLAGS_BUTTON2)
+    button = BTN_RIGHT;
+  else if (flags & PTR_FLAGS_BUTTON3)
+    button = BTN_MIDDLE;
+  else if (flags & PTR_XFLAGS_BUTTON1)
+    button = BTN_SIDE;
+  else if (flags & PTR_XFLAGS_BUTTON2)
+    button = BTN_EXTRA;
+
+  if (button)
+    {
+      grd_rdp_event_queue_add_input_event_pointer_button (rdp_event_queue,
+                                                          button, button_state);
+    }
+
+  return TRUE;
+}
+
 static gboolean
 is_pause_key_sequence (GrdSessionRdp *session_rdp,
                        uint16_t       vkcode,
@@ -989,6 +1033,8 @@ rdp_peer_post_connect (freerdp_peer *peer)
   RdpPeerContext *rdp_peer_context = (RdpPeerContext *) peer->context;
   GrdSessionRdp *session_rdp = rdp_peer_context->session_rdp;
   rdpSettings *rdp_settings = peer->context->settings;
+  uint32_t rdp_version =
+    freerdp_settings_get_uint32 (rdp_settings, FreeRDP_RdpVersion);
   uint32_t keyboard_type =
     freerdp_settings_get_uint32 (rdp_settings, FreeRDP_KeyboardType);
   uint32_t os_major_type =
@@ -1008,6 +1054,7 @@ rdp_peer_post_connect (freerdp_peer *peer)
   g_debug ("New RDP client: [OS major type, OS minor type]: [%s, %s]",
            freerdp_peer_os_major_type_string (peer),
            freerdp_peer_os_minor_type_string (peer));
+  g_debug ("[RDP] Maximum common RDP version 0x%08X", rdp_version);
   g_debug ("[RDP] Client uses keyboard type %u", keyboard_type);
 
   g_debug ("[RDP] Virtual Channels: compression flags: %u, "
@@ -1077,7 +1124,7 @@ rdp_peer_context_free (freerdp_peer   *peer,
   if (!rdp_peer_context)
     return;
 
-  g_clear_object (&rdp_peer_context->rdp_dvc);
+  g_clear_object (&rdp_peer_context->dvc_handler);
 
   if (rdp_peer_context->vcm != INVALID_HANDLE_VALUE)
     g_clear_pointer (&rdp_peer_context->vcm, WTSCloseServer);
@@ -1127,8 +1174,8 @@ rdp_peer_context_new (freerdp_peer   *peer,
       return FALSE;
     }
 
-  rdp_peer_context->rdp_dvc = grd_rdp_dvc_new (rdp_peer_context->vcm,
-                                               &rdp_peer_context->rdp_context);
+  rdp_peer_context->dvc_handler =
+    grd_rdp_dvc_handler_new (rdp_peer_context->vcm);
 
   return TRUE;
 }
@@ -1271,7 +1318,7 @@ init_rdp_session (GrdSessionRdp  *session_rdp,
 
   freerdp_settings_set_bool (rdp_settings, FreeRDP_HasExtendedMouseEvent, TRUE);
   freerdp_settings_set_bool (rdp_settings, FreeRDP_HasHorizontalWheel, TRUE);
-  freerdp_settings_set_bool (rdp_settings, FreeRDP_HasRelativeMouseEvent, FALSE);
+  freerdp_settings_set_bool (rdp_settings, FreeRDP_HasRelativeMouseEvent, TRUE);
   freerdp_settings_set_bool (rdp_settings, FreeRDP_HasQoeEvent, FALSE);
   freerdp_settings_set_bool (rdp_settings, FreeRDP_UnicodeInput, TRUE);
 
@@ -1289,6 +1336,7 @@ init_rdp_session (GrdSessionRdp  *session_rdp,
   rdp_input->SynchronizeEvent = rdp_input_synchronize_event;
   rdp_input->MouseEvent = rdp_input_mouse_event;
   rdp_input->ExtendedMouseEvent = rdp_input_extended_mouse_event;
+  rdp_input->RelMouseEvent = rdp_input_rel_mouse_event;
   rdp_input->KeyboardEvent = rdp_input_keyboard_event;
   rdp_input->UnicodeKeyboardEvent = rdp_input_unicode_keyboard_event;
 
@@ -1367,11 +1415,12 @@ socket_thread_func (gpointer data)
       if (peer->connected &&
           WTSVirtualChannelManagerIsChannelJoined (vcm, DRDYNVC_SVC_CHANNEL_NAME))
         {
-          GrdRdpTelemetry *telemetry;
-          GrdRdpGraphicsPipeline *graphics_pipeline;
-          GrdRdpAudioPlayback *audio_playback;
-          GrdRdpDisplayControl *display_control;
-          GrdRdpAudioInput *audio_input;
+          GrdRdpDvcTelemetry *telemetry;
+          GrdRdpDvcGraphicsPipeline *graphics_pipeline;
+          GrdRdpDvcInput *input;
+          GrdRdpDvcAudioPlayback *audio_playback;
+          GrdRdpDvcDisplayControl *display_control;
+          GrdRdpDvcAudioInput *audio_input;
 
           switch (WTSVirtualChannelManagerGetDrdynvcState (vcm))
             {
@@ -1386,20 +1435,23 @@ socket_thread_func (gpointer data)
               g_mutex_lock (&rdp_peer_context->channel_mutex);
               telemetry = rdp_peer_context->telemetry;
               graphics_pipeline = rdp_peer_context->graphics_pipeline;
+              input = rdp_peer_context->input;
               audio_playback = rdp_peer_context->audio_playback;
               display_control = rdp_peer_context->display_control;
               audio_input = rdp_peer_context->audio_input;
 
               if (telemetry && !session_rdp->session_should_stop)
-                grd_rdp_telemetry_maybe_init (telemetry);
+                grd_rdp_dvc_maybe_init (GRD_RDP_DVC (telemetry));
               if (graphics_pipeline && !session_rdp->session_should_stop)
-                grd_rdp_graphics_pipeline_maybe_init (graphics_pipeline);
+                grd_rdp_dvc_maybe_init (GRD_RDP_DVC (graphics_pipeline));
+              if (input && !session_rdp->session_should_stop)
+                grd_rdp_dvc_maybe_init (GRD_RDP_DVC (input));
               if (audio_playback && !session_rdp->session_should_stop)
-                grd_rdp_audio_playback_maybe_init (audio_playback);
+                grd_rdp_dvc_maybe_init (GRD_RDP_DVC (audio_playback));
               if (display_control && !session_rdp->session_should_stop)
-                grd_rdp_display_control_maybe_init (display_control);
+                grd_rdp_dvc_maybe_init (GRD_RDP_DVC (display_control));
               if (audio_input && !session_rdp->session_should_stop)
-                grd_rdp_audio_input_maybe_init (audio_input);
+                grd_rdp_dvc_maybe_init (GRD_RDP_DVC (audio_input));
               g_mutex_unlock (&rdp_peer_context->channel_mutex);
               break;
             }
@@ -1568,6 +1620,7 @@ grd_session_rdp_stop (GrdSession *session)
   g_clear_object (&rdp_peer_context->clipboard_rdp);
   g_clear_object (&rdp_peer_context->audio_playback);
   g_clear_object (&rdp_peer_context->display_control);
+  g_clear_object (&rdp_peer_context->input);
   g_clear_object (&rdp_peer_context->graphics_pipeline);
   g_clear_object (&rdp_peer_context->telemetry);
   g_mutex_unlock (&rdp_peer_context->channel_mutex);
@@ -1617,29 +1670,29 @@ initialize_graphics_pipeline (GrdSessionRdp *session_rdp)
 {
   rdpContext *rdp_context = session_rdp->peer->context;
   RdpPeerContext *rdp_peer_context = (RdpPeerContext *) rdp_context;
-  GrdRdpGraphicsPipeline *graphics_pipeline;
-  GrdRdpTelemetry *telemetry;
+  GrdRdpDvcGraphicsPipeline *graphics_pipeline;
+  GrdRdpDvcTelemetry *telemetry;
 
   g_assert (!rdp_peer_context->telemetry);
   g_assert (!rdp_peer_context->graphics_pipeline);
 
-  telemetry = grd_rdp_telemetry_new (session_rdp,
-                                     rdp_peer_context->rdp_dvc,
-                                     rdp_peer_context->vcm,
-                                     rdp_context);
+  telemetry = grd_rdp_dvc_telemetry_new (session_rdp,
+                                         rdp_peer_context->dvc_handler,
+                                         rdp_peer_context->vcm,
+                                         rdp_context);
   rdp_peer_context->telemetry = telemetry;
 
   graphics_pipeline =
-    grd_rdp_graphics_pipeline_new (session_rdp,
-                                   session_rdp->renderer,
-                                   rdp_peer_context->rdp_dvc,
-                                   rdp_peer_context->vcm,
-                                   rdp_context,
-                                   rdp_peer_context->network_autodetection,
-                                   rdp_peer_context->encode_stream,
-                                   rdp_peer_context->rfx_context);
-  grd_rdp_graphics_pipeline_set_hwaccel_nvidia (graphics_pipeline,
-                                                session_rdp->hwaccel_nvidia);
+    grd_rdp_dvc_graphics_pipeline_new (session_rdp,
+                                       session_rdp->renderer,
+                                       rdp_peer_context->dvc_handler,
+                                       rdp_peer_context->vcm,
+                                       rdp_context,
+                                       rdp_peer_context->network_autodetection,
+                                       rdp_peer_context->encode_stream,
+                                       rdp_peer_context->rfx_context);
+  grd_rdp_dvc_graphics_pipeline_set_hwaccel_nvidia (graphics_pipeline,
+                                                    session_rdp->hwaccel_nvidia);
   rdp_peer_context->graphics_pipeline = graphics_pipeline;
 }
 
@@ -1649,17 +1702,21 @@ initialize_remaining_virtual_channels (GrdSessionRdp *session_rdp)
   rdpContext *rdp_context = session_rdp->peer->context;
   RdpPeerContext *rdp_peer_context = (RdpPeerContext *) rdp_context;
   rdpSettings *rdp_settings = rdp_context->settings;
-  GrdRdpDvc *rdp_dvc = rdp_peer_context->rdp_dvc;
+  GrdRdpDvcHandler *dvc_handler = rdp_peer_context->dvc_handler;
   HANDLE vcm = rdp_peer_context->vcm;
 
+  rdp_peer_context->input =
+    grd_rdp_dvc_input_new (session_rdp->layout_manager,
+                           session_rdp, dvc_handler, vcm);
+
   if (session_rdp->screen_share_mode == GRD_RDP_SCREEN_SHARE_MODE_EXTEND)
     {
       rdp_peer_context->display_control =
-        grd_rdp_display_control_new (session_rdp->layout_manager,
-                                     session_rdp,
-                                     rdp_peer_context->rdp_dvc,
-                                     rdp_peer_context->vcm,
-                                     get_max_monitor_count (session_rdp));
+        grd_rdp_dvc_display_control_new (session_rdp->layout_manager,
+                                         session_rdp,
+                                         dvc_handler,
+                                         rdp_peer_context->vcm,
+                                         get_max_monitor_count (session_rdp));
     }
   if (WTSVirtualChannelManagerIsChannelJoined (vcm, CLIPRDR_SVC_CHANNEL_NAME))
     {
@@ -1675,12 +1732,14 @@ initialize_remaining_virtual_channels (GrdSessionRdp *session_rdp)
       !freerdp_settings_get_bool (rdp_settings, FreeRDP_RemoteConsoleAudio))
     {
       rdp_peer_context->audio_playback =
-        grd_rdp_audio_playback_new (session_rdp, rdp_dvc, vcm, rdp_context);
+        grd_rdp_dvc_audio_playback_new (session_rdp, dvc_handler, vcm,
+                                        rdp_context);
     }
   if (freerdp_settings_get_bool (rdp_settings, FreeRDP_AudioCapture))
     {
       rdp_peer_context->audio_input =
-        grd_rdp_audio_input_new (session_rdp, rdp_dvc, vcm, rdp_context);
+        grd_rdp_dvc_audio_input_new (session_rdp, dvc_handler, vcm,
+                                     rdp_context);
     }
 }
 
@@ -1707,7 +1766,8 @@ grd_session_rdp_remote_desktop_session_started (GrdSession *session)
   GrdSessionRdp *session_rdp = GRD_SESSION_RDP (session);
   rdpContext *rdp_context = session_rdp->peer->context;
   RdpPeerContext *rdp_peer_context = (RdpPeerContext *) rdp_context;
-  GrdRdpGraphicsPipeline *graphics_pipeline = rdp_peer_context->graphics_pipeline;
+  GrdRdpDvcGraphicsPipeline *graphics_pipeline =
+    rdp_peer_context->graphics_pipeline;
 
   if (!grd_rdp_renderer_start (session_rdp->renderer,
                                session_rdp->hwaccel_vulkan,
@@ -1722,7 +1782,9 @@ grd_session_rdp_remote_desktop_session_started (GrdSession *session)
                                                    GRD_RDP_PHASE_SESSION_STARTED);
   grd_rdp_layout_manager_notify_session_started (session_rdp->layout_manager,
                                                  session_rdp->cursor_renderer,
-                                                 rdp_context);
+                                                 rdp_context,
+                                                 session_rdp->screen_share_mode);
+  grd_rdp_event_queue_flush_synchronization (session_rdp->rdp_event_queue);
 }
 
 static void
