@@ -18,8 +18,7 @@
  *
  */
 
-#ifndef GRD_SETTINGS_USER_H
-#define GRD_SETTINGS_USER_H
+#pragma once
 
 #include "grd-settings.h"
 
@@ -27,6 +26,4 @@
 G_DECLARE_FINAL_TYPE (GrdSettingsUser, grd_settings_user,
                       GRD, SETTINGS_USER, GrdSettings)
 
-GrdSettingsUser *grd_settings_user_new (GrdRuntimeMode runtime_mode);
-
-#endif /* GRD_SETTINGS_USER_H */
+GrdSettingsUser *grd_settings_user_new (void);
